@@ -9,7 +9,7 @@ import numpy as np
 import math
 
 import gym
-from navsim import DDPGAgent
+from navsim.agent import DDPGAgent
 from navsim.util import sizeof_fmt, image_layout, s_hwc_to_chw
 from navsim.util import ObjDict, ResourceCounter
 
@@ -60,7 +60,7 @@ class Executor:
         """
         self.run_id = run_id
 
-        self.conf = conf
+        #self.conf = conf
         self.resume = resume
 
         self.run_base_folder = Path(self.run_id).resolve()
@@ -76,11 +76,11 @@ class Executor:
         else:
 
             self.run_base_folder.mkdir(parents=True, exist_ok=True)
-            self.conf.save_to_json_file(f"{self.run_base_folder_str}/conf.json")
+            conf.save_to_json_file(f"{self.run_base_folder_str}/conf.json")
             self.file_mode = 'w+'
 
-        self.run_conf = ObjDict(self.conf.run_config)
-        self.env_config = ObjDict(self.conf.env_config)
+        self.run_conf = ObjDict(conf.run_config)
+        self.env_config = ObjDict(conf.env_config)
 
         pylog_filename = self.run_base_folder / 'py.log'  # TODO: use logger
         self.pylog_filename = str(pylog_filename)
@@ -104,7 +104,7 @@ class Executor:
                 folder.mkdir(parents=True, exist_ok=True)
 
         self.agent_folder_str = str(agent_folder)
-        self.env_config.log_folder = str(env_log_folder)
+        self.env_config["log_folder"] = str(env_log_folder)
         self.summary_writer = SummaryWriter(f"{str(tb_folder)}")
 
         self.rc = ResourceCounter()
@@ -115,9 +115,9 @@ class Executor:
                 with open(self.episode_num_filename,
                           mode='r') as episode_num_file:
                     episode_num = int(episode_num_file.read())
-                    self.conf.env_config["start_from_episode"] = episode_num + 1
+                    self.env_config["start_from_episode"] = episode_num + 1
             else:
-                self.conf.env_config["start_from_episode"] = 1
+                self.env_config["start_from_episode"] = 1
             self.env = None
             self.env_open()
 
@@ -142,7 +142,7 @@ class Executor:
 
             if self.run_conf["mem_backend"] == "cupy":
                 if torch.cuda.is_available():
-                    from navsim import CupyMemory
+                    from navsim.memory import CupyMemory
                     CupyMemory.set_device(self.run_conf['agent_gpu_id'])
                     mem_backend = CupyMemory
                 else:
@@ -151,7 +151,7 @@ class Executor:
             elif self.run_conf["mem_backend"] == "numpy":
                 if torch.cuda.is_available():
                     print("Warning: GPU is available but mem_backend=numpy")
-                from navsim import NumpyMemory
+                from navsim.memory import NumpyMemory
                 mem_backend = NumpyMemory
 
             if resume and self.run_base_folder.is_dir():
@@ -254,7 +254,7 @@ class Executor:
     def env_open(self):
         self.rc.start()
 
-        self.env = gym.make(self.conf["env"], env_config=self.env_config)
+        self.env = gym.make(self.run_conf.env, env_config=self.env_config)
         time_since_start, current_memory, peak_memory = self.rc.stop()
         log_str = f'Unity env creation resource usage: \n' \
                   f'time:{time_since_start},' \
@@ -281,7 +281,7 @@ class Executor:
         t_max = int(self.run_conf['episode_max_steps'])
         total_episodes = int(self.run_conf['total_episodes'])
         num_episodes = total_episodes - (
-                self.conf.env_config["start_from_episode"] - 1)
+                self.env_config["start_from_episode"] - 1)
         train_interval = int(self.run_conf['train_interval'])
         checkpoint_interval = int(self.run_conf['checkpoint_interval'])
         num_episode_blocks = int(math.ceil(num_episodes / checkpoint_interval))
@@ -289,7 +289,7 @@ class Executor:
         batch_size = self.run_conf['batch_size']
         # save the state json at start of run
         model_filename = f"{self.run_base_folder_str}/" \
-                         f"{self.conf.env_config['start_from_episode']}_" \
+                         f"{self.env_config['start_from_episode']}_" \
                          f"{total_episodes}_start_agent_state.json"
         json.dump(self.agent.get_state_dict(), open(model_filename, 'w'),
                   indent=2, sort_keys=True, cls=TorchJSONEncoder)
@@ -308,14 +308,14 @@ class Executor:
         step_loss = np.full((checkpoint_interval, t_max, 2), 0.0)
 
         for i in range(0, num_episode_blocks):
-            start_episode = (self.conf.env_config["start_from_episode"] - 1) + (
+            start_episode = (self.env_config["start_from_episode"] - 1) + (
                     (i * checkpoint_interval) + 1)
-            stop_episode = min((self.conf.env_config["start_from_episode"] - 1)
+            stop_episode = min((self.env_config["start_from_episode"] - 1)
                                + ((i + 1) * checkpoint_interval),
                                total_episodes)
             episodes_in_block = stop_episode - start_episode + 1
             # episode_rewards = np.full(episodes_in_block, 0.0)
-            ckpt_ctr = 0
+            ckpt_e = 0
             ckpt_t_global = t_global
 
             for episode_num in tqdm(
@@ -325,12 +325,11 @@ class Executor:
                     bar_format='{l_bar}{bar}| {n_fmt}/{total_fmt} [{rate_fmt}{postfix}]'):
                 self.rc.start()
                 episode_counter = episode_num - start_episode
-                episode_resources[
-                    ckpt_ctr, 0] = self.rc.snapshot()  # e0
+                episode_resources[ckpt_e, 0] = self.rc.snapshot()  # e0
 
                 # initialise the episode counters
                 episode_done = False
-                # episode_reward = 0
+                # episode_return = 0
                 t = 1
 
                 # self.env.start_navigable_map()
@@ -356,35 +355,35 @@ class Executor:
                     # s5,6: before and after env.step()
                     # s7,8: before and after memory.append()
                     # s9: end of step
-                    step_res[ckpt_ctr, t - 1, 0] = self.rc.snapshot()  # s0
+                    step_res[ckpt_e, t - 1, 0] = self.rc.snapshot()  # s0
 
                     # do the random sampling until enough memory is full
                     if self.memory.size < batch_size:
                         a = self.env.action_space.sample()
                         # rescale break between 0,1 from -1,1
                         # NewValue = (((OldValue - OldMin) * (NewMax - NewMin)) / (OldMax - OldMin)) + NewMin
-                        a[2] = (a[2] + 1) / 2
-                        step_res[ckpt_ctr, t - 1, 1:5] = [[0.0] * 3] * 4
+                        # a[2] = (a[2] + 1) / 2
+                        step_res[ckpt_e, t - 1, 1:5] = [[0.0] * 3] * 4
                         # s1-4
                     else:
                         # TODO: Find the best place to train, moved here for now
                         if train_interval and (
                                 (t_global % train_interval) == 0):
-                            step_res[ckpt_ctr, t - 1, 1] = self.rc.snapshot()
+                            step_res[ckpt_e, t - 1, 1] = self.rc.snapshot()
                             # s1
                             batch_s, batch_a, batch_r, batch_s_, batch_d = \
                                 self.memory.sample(batch_size)
-                            step_res[ckpt_ctr, t - 1, 2] = self.rc.snapshot()
+                            step_res[ckpt_e, t - 1, 2] = self.rc.snapshot()
                             # s2
                             # print('training the agent')
-                            step_res[ckpt_ctr, t - 1, 3] = self.rc.snapshot()
+                            step_res[ckpt_e, t - 1, 3] = self.rc.snapshot()
                             # s3
                             self.agent.train(batch_s, batch_a, batch_r,
                                              batch_s_, batch_d)
-                            step_res[ckpt_ctr, t - 1, 4] = self.rc.snapshot()
+                            step_res[ckpt_e, t - 1, 4] = self.rc.snapshot()
                             # s4
                         else:
-                            step_res[ckpt_ctr, t - 1, 1:5] = [[0.0] * 3] * 4
+                            step_res[ckpt_e, t - 1, 1:5] = [[0.0] * 3] * 4
                             # s1-4
 
                         a = (self.agent.select_action(s) + np.random.normal(
@@ -395,9 +394,9 @@ class Executor:
                             self.max_action
                         )
 
-                    step_res[ckpt_ctr, t - 1, 5] = self.rc.snapshot()  # s5
+                    step_res[ckpt_e, t - 1, 5] = self.rc.snapshot()  # s5
                     s_, r, episode_done, info = self.env.step(a)
-                    step_res[ckpt_ctr, t - 1, 6] = self.rc.snapshot()  # s6
+                    step_res[ckpt_e, t - 1, 6] = self.rc.snapshot()  # s6
 
                     # because pytorch can only deal with images in CHW format
                     # we are making the optimization here to convert from HWC to CHW format.
@@ -410,30 +409,30 @@ class Executor:
                     # s_ = [s_]
                     # for item in s_:
 
-                    step_res[ckpt_ctr, t - 1, 7] = self.rc.snapshot()
+                    step_res[ckpt_e, t - 1, 7] = self.rc.snapshot()
                     # s7
                     self.memory.append(
                         s=s, a=a, s_=s_, r=r,
                         d=float(episode_done))  # if t < t_max -1 else 1)
                     s = s_
-                    step_res[ckpt_ctr, t - 1, 8] = self.rc.snapshot()  # s8
+                    step_res[ckpt_e, t - 1, 8] = self.rc.snapshot()  # s8
 
-                    step_rew[ckpt_ctr, t - 1] = r
-                    step_spl[ckpt_ctr, t - 1] = self.env.spl_current
-                    step_loss[ckpt_ctr, t - 1] = [
+                    step_rew[ckpt_e, t - 1] = r
+                    step_spl[ckpt_e, t - 1] = self.env.spl_current
+                    step_loss[ckpt_e, t - 1] = [
                         0 if self.agent.actor_loss is None else self.agent.actor_loss.data.cpu().numpy(),
                         0 if self.agent.critic_loss is None else self.agent.critic_loss.data.cpu().numpy()]
 
-                    # print(step_loss[ckpt_ctr, t - 1])
-                    # episode_rewards[ckpt_ctr] += r
+                    # print(step_loss[ckpt_e, t - 1])
+                    # episode_rewards[ckpt_e] += r
 
                     # if self.memory.size >= self.run_conf['batch_size'] * self.run_conf['batches_before_train']:
 
                     #                    if (t >= self.config['batch_size'] * self.config['batches_before_train']) and (t % 1000 == 0):
                     # episode_evaluations.append(evaluate_policy(self.agent, self.env, self.config['seed']))
 
-                    step_res[ckpt_ctr, t - 1, 9] = self.rc.snapshot()  # s9
-                    # print(ckpt_ctr,t-1, step_res[ckpt_ctr][t-1])
+                    step_res[ckpt_e, t - 1, 9] = self.rc.snapshot()  # s9
+                    # print(ckpt_e,t-1, step_res[ckpt_e][t-1])
                     # print(t,step_res[0][0])
                     if episode_done or (t_max and (t >= t_max)):
                         break
@@ -443,17 +442,17 @@ class Executor:
                 # print(step_res[0][0])
                 # end of while loop for one episode
                 # episode end processing
-                episode_resources[ckpt_ctr, 1] = self.rc.stop()  # e1
-                episode_steps[ckpt_ctr] = t
-                ckpt_ctr += 1
+                episode_resources[ckpt_e, 1] = self.rc.stop()  # e1
+                episode_steps[ckpt_e] = t
+                ckpt_e += 1
 
                 #            if self.enable_logging:
-                #                self.writer.add_scalar('Episode Reward', episode_reward, t)
-                #            episode_rewards.append(episode_reward)
+                #                self.writer.add_scalar('Episode Reward', episode_return, t)
+                #            episode_rewards.append(episode_return)
 
                 # s = self.env.reset()[0]
                 # episode_done = False
-                # episode_reward = 0
+                # episode_return = 0
                 # episode_timesteps = 0
                 # episode_num += 1
             # end of tqdm-for loop - checkpoint block of episodes
@@ -473,15 +472,21 @@ class Executor:
             for e_num in range(episodes_in_block):
                 episode_time = episode_resources[e_num, 1, 0] - \
                                episode_resources[e_num, 0, 0]
-                episode_reward = step_rew[e_num, 0:episode_steps[e_num]].sum()
+                episode_return = step_rew[e_num, 0:episode_steps[e_num]].sum()
                 self.write_tb('episode',
-                              {'reward': episode_reward,
+                              {'return': episode_return,
                                'time': episode_time,
-                               'peak_memory': episode_resources[e_num, 1, 2]},
+                               'peak_memory': episode_resources[e_num, 1, 2],
+                               'total_steps': episode_steps[e_num],
+                               'max_spl': step_spl[e_num].max(),
+                               'min_spl': step_spl[e_num].min(),
+                               'spl_span': step_spl[e_num].max() - step_spl[
+                                   e_num].min()
+                               },
                               start_episode + e_num)  # episode_num
                 self.episode_results_writer.writerow(
                     [start_episode + e_num,  # episode_num
-                     episode_reward,
+                     episode_return,
                      episode_time,
                      episode_resources[e_num, 1, 2]])  # episode_peak_memory
 
@@ -538,10 +543,10 @@ class Executor:
             # episodes checkpoint loop finishes
             self.episode_results_file.flush()
             self.step_results_file.flush()
-            ckpt_ctr = 0
+            ckpt_e = 0
         # save the state json at end of run
         model_filename = f"{self.run_base_folder_str}/" \
-                         f"{self.conf.env_config['start_from_episode']}_" \
+                         f"{self.env_config['start_from_episode']}_" \
                          f"{total_episodes}_stop_agent_state.json"
         json.dump(self.agent.get_state_dict(), open(model_filename, 'w'),
                   indent=2, sort_keys=True, cls=TorchJSONEncoder)
